@@ -9,6 +9,7 @@ import numpy as np
 from .system import System as _System
 from .system import _SingleDoFCoordinatesDict
 from .nonlinear_systems import MultiDoFNonLinearSystem as _MDNLS
+from .functions import benchmark_par_to_canonical
 
 
 class _LinearSystem(_System):
@@ -823,8 +824,9 @@ class MultiDoFLinearSystem(_MDNLS):
         >>> sys.constants['To'] = 1.0  # Nm
         >>> sys.constants['beta'] = 0.01  # rad/s
         >>> def forcing(To, beta, time):
-        ...     return np.array([[To * np.cos(beta * time)],
-        ...                      [To * np.sin(beta * time)]])
+        ...     T1 = To * np.cos(beta * time)
+        ...     T2 = To * np.sin(beta * time)
+        ...     return T1, T2
         ...
         >>> sys.forcing_func = forcing
 
@@ -833,7 +835,21 @@ class MultiDoFLinearSystem(_MDNLS):
 
     @forcing_func.setter
     def forcing_func(self, func):
+        args = [self._get_par_vals(k) for k in getargspec(func).args]
+        res = func(*args)
+        msg = ("Forcing function must return the same number of values as "
+               "the number of coordinates.")
+        try:
+            len(res)
+        except TypeError:  # returns a single value
+            if len(self.coordinates) != 1:
+                raise ValueError(msg)
+        else:
+            if len(res) != len(self.coordinates):
+                raise ValueError(msg)
+
         self._forcing_func = func
+        self._forcing_func_arg_names = getargspec(func).args
 
     def canonical_coefficients(self):
         """Returns the mass, damping, and stiffness matrices in that order."""
@@ -874,17 +890,29 @@ class MultiDoFLinearSystem(_MDNLS):
 
         return A, B
 
-    def _eval_forcing(self):
+    def _eval_forcing(self, t):
+        # t is either:
         # shape(2n, 1)
-        u = np.zeros((len(self.states), 1))
+        # shape(m, 2n, 1)
+        size_t = np.size(t)
+        len_states = len(self.coordinates) + len(self.speeds)
+        if size_t > 1:
+            u = np.zeros((len(t), len_states, 1))
+        else:
+            u = np.zeros((len_states, 1))
+
         if self._compute_forcing:
-            arg_names = getargspec(self.forcing_func).args
-            arg_vals = [self._get_par_vals(k) for k in arg_names]
+            self._time['t'] = t
+            arg_vals = [self._get_par_vals(k) for k in
+                        self._forcing_func_arg_names]
             f = self.forcing_func(*arg_vals)
-            if len(f.shape) == 1:
-                u[len(self.coordinates):, 0] = f
-            else:
-                u[len(self.coordinates):] = f
+            for i, fi in enumerate(f):
+                if np.size(fi) == 1 and size_t > 1:
+                    fi = np.repeat(fi, len(t))
+                if size_t > 1:
+                    u[:, len(self.coordinates) + i, 0] = fi
+                else:
+                    u[len(self.coordinates) + i, 0] = fi
             return u
         else:
             return u
@@ -894,8 +922,10 @@ class MultiDoFLinearSystem(_MDNLS):
         A, B = self._form_A_B()
 
         def eval_rhs(x, t):
-            u = self._eval_forcing()
+            u = self._eval_forcing(t)
+            # This matrix mul is one of two forms:
             # (2n x 2n) * (2n x 1) + (2n x 2n) * (2n x 1)
+            # (2n x 2n) * (m x 2n x 1) + (2n x 2n) * (m x 2n x 1)
             return A @ x + B @ u
 
         return eval_rhs
@@ -937,6 +967,7 @@ class MultiDoFLinearSystem(_MDNLS):
         traj = self.free_response(final_time, initial_time=initial_time,
                                   sample_rate=sample_rate, **kwargs)
         self._compute_forcing = False
+        self._time['t'] = 0.0
         return traj
 
 
@@ -1707,3 +1738,147 @@ class BallChannelPendulumSystem(MultiDoFLinearSystem):
             trough.set_center((trough_x, trough_y))
 
         self.config_plot_update_func = update
+
+
+class FourStoryBuildingSystem(MultiDoFLinearSystem):
+
+    def __init__(self):
+
+        super(FourStoryBuildingSystem, self).__init__()
+
+        self.constants['m1'] = 4000  # kg
+        self.constants['m2'] = 4000  # kg
+        self.constants['m3'] = 4000  # kg
+        self.constants['m4'] = 4000  # kg
+        self.constants['k1'] = 5000  # N/m
+        self.constants['k2'] = 5000  # N/m
+        self.constants['k3'] = 5000  # N/m
+        self.constants['k4'] = 5000  # N/m
+
+        self.coordinates['x1'] = 0.001  # m
+        self.coordinates['x2'] = 0.010  # m
+        self.coordinates['x3'] = 0.020  # m
+        self.coordinates['x4'] = 0.025  # m
+
+        self.speeds['v1'] = 0.0  # m/s
+        self.speeds['v2'] = 0.0  # m/s
+        self.speeds['v3'] = 0.0  # m/s
+        self.speeds['v4'] = 0.0  # m/s
+
+        def can_coeffs_matrices(m1, m2, m3, m4, k1, k2, k3, k4):
+            M = np.diag([m1, m2, m3, m4])
+            C = np.zeros((4, 4))
+            K = np.array([[k1 + k2, -k2, 0, 0],
+                          [-k2, k2 + k3, -k3, 0],
+                          [0, -k3, k3 + k4, -k4],
+                          [0, 0, -k4, k4]])
+            return M, C, K
+
+        self.canonical_coeffs_func = can_coeffs_matrices
+
+        width = 3  # meters
+        height = 0.5  # meters
+        scale = 20.0  # magnify the motion
+
+        def plot_config(x1, x2, x3, x4, time):
+
+            fig, ax = plt.subplots(1, 1)
+
+            xi = scale * np.array([x1, x2, x3, x4])
+
+            ax.set_ylim((0, 14))
+            ax.set_xlim((-5, 5))
+
+            # create a rectangle for each floor
+            rects = []
+            for i in range(4):
+                rect = Rectangle((-width / 2 + xi[i], 3 - height + i * 3),
+                                 width, height, fill=False)
+                rects.append(rect)
+                ax.add_patch(rect)
+
+            # make some vertical lines for the right and left walls
+            left_walls = ax.plot(-width / 2 * np.ones(5) + np.hstack((0, xi)),
+                                 [0, 3, 6, 9, 12], color='blue')[0]
+            right_walls = ax.plot(width / 2 * np.ones(5) + np.hstack((0, xi)),
+                                  [0, 3, 6, 9, 12], color='blue')[0]
+
+            text = ax.text(-4.0, 13.0, 'Time = {:0.3f} s'.format(time))
+
+            ax.set_aspect('equal')
+            ax.set_xlabel('Distance [m]')
+            ax.set_ylabel('Distance [m]')
+            ax.set_title('Lateral Motion Magnified by {:1.0f}X'.format(scale))
+
+            #xticks = ax.get_xticks()
+            #labels = ['{:1.2}'.format(v) for v in xticks / scale]
+            #ax.set_xticklabels(labels)
+
+            return fig, rects, left_walls, right_walls, text
+
+        self.config_plot_func = plot_config
+
+        def update(x1, x2, x3, x4, time, rects, left_walls, right_walls, text):
+            # grab the ith coordinate vector
+            xi = scale * np.array([x1, x2, x3, x4])
+
+            # move each rectangle laterally by the associated coordinate
+            for i, rect in enumerate(rects):
+                rect.set_xy([-width / 2 + xi[i], 3 - height + i * 3])
+
+            left_walls.set_xdata(-width / 2 * np.ones(5) + np.hstack((0, xi)))
+            right_walls.set_xdata(width / 2 * np.ones(5) + np.hstack((0, xi)))
+
+            text.set_text('Time = {:0.3f} s'.format(time))
+
+        self.config_plot_update_func = update
+
+class BicycleSystem(MultiDoFLinearSystem):
+
+    def __init__(self):
+
+        super(BicycleSystem, self).__init__()
+
+        self.constants['v'] = 4.6
+        self.constants['w'] = 1.02
+        self.constants['c'] = 0.08
+        self.constants['lam'] = np.pi / 10.
+        self.constants['g'] = 9.81
+        self.constants['rR'] = 0.3
+        self.constants['mR'] = 2.0
+        self.constants['IRxx'] = 0.0603
+        self.constants['IRyy'] = 0.12
+        self.constants['xB'] = 0.3
+        self.constants['zB'] = -0.9
+        self.constants['mB'] = 85.0
+        self.constants['IBxx'] = 9.2
+        self.constants['IByy'] = 11.0
+        self.constants['IBzz'] = 2.8
+        self.constants['IBxz'] = 2.4
+        self.constants['xH'] = 0.9
+        self.constants['zH'] = -0.7
+        self.constants['mH'] = 4.0
+        self.constants['IHxx'] = 0.05892
+        self.constants['IHyy'] = 0.06
+        self.constants['IHzz'] = 0.00708
+        self.constants['IHxz'] = -0.00756
+        self.constants['rF'] = 0.35
+        self.constants['mF'] = 3.0
+        self.constants['IFxx'] = 0.1405
+        self.constants['IFyy'] = 0.28
+
+        self.coordinates['phi'] = 0.0
+        self.coordinates['delta'] = 0.0
+
+        self.speeds['phidot'] = 0.5
+        self.speeds['deltadot'] = 0.0
+
+        def can_coeff_matrices(v, w, c, lam, g, rR, mR, IRxx, IRyy, xB, zB, mB,
+                               IBxx, IByy, IBzz, IBxz, xH, zH, mH, IHxx, IHyy,
+                               IHzz, IHxz, rF, mF, IFxx, IFyy):
+
+            M, C1, K0, K2 = benchmark_par_to_canonical(self.constants)
+
+            return M, v * C1, g * K0 + v**2 * K2
+
+        self.canonical_coeffs_func = can_coeff_matrices
