@@ -23,30 +23,35 @@ class MultiDoFNonLinearSystem(_System):
 
     @property
     def diff_eq_func(self):
-        """A function that returns the time derivatives of the coordinate and
-        speed, i.e. computes the right hand side of the two order differential
-        equations. This equation looks like the following for linear motion:
+        """A function that returns the time derivatives of the coordinates and
+        speeds, i.e. computes the right hand side of the explicit first order
+        differential equations. This equation looks like the following for
+        linear motion::
 
-            dy
+            dx
             -- = f(t, q1, ..., qn, u1, ..., un, p1, p2, ..., pO)
             dt
 
         where:
 
-            - t: a time value
-            - q: the coordinates
-            - u: the speeds
-            - p: any number of constants or measurements, O is the number of
-              constants.
+        - x: [q1, ..., qn, u1, ..., un], the "state vector"
+        - t: a time value
+        - q: the coordinates
+        - u: the speeds
+        - p: any number of constants, O is the number of constants
 
-        Notes
-        =====
         Your function should be able to operate on 1d arrays as inputs, i.e.
         use numpy math functions in your function, e.g. ``numpy.sin`` instead
-        of ``math.sin``.
+        of ``math.sin``. Besides the constants, coordinates, and speeds, there
+        is a special variable ``time`` that you can use to give the current
+        value of time inside your function.
 
-        **The function has to return the derivatives of the states in the order
-        of the ``state`` attribute.**
+        .. note:: The function has to return the derivatives of the states in
+           the order of the ``state`` attribute.
+
+        .. warning:: Do not use measurements as a function argument. This may
+           cause causality issues and is not yet supported. You are unlikely to
+           get a correct answer if you use a measurement in this function.
 
         Example
         =======
@@ -54,15 +59,18 @@ class MultiDoFNonLinearSystem(_System):
         >>> sys.constants['gravity'] = 9.8  # m/s**2
         >>> sys.constants['length'] = 1.0  # m
         >>> sys.constants['mass'] = 0.5  # kg
+        >>> sys.constants['omega_b'] = 0.1  # rad/s
         >>> sys.coordinates['theta'] = 0.3  # rad
         >>> sys.speeds['omega'] = 0.0  # rad/s
         >>> sys.states
         {'theta': 0.3, 'omega': 0.0}  # note the order!
-        >>> def rhs(theta, omega, gravity, length, mass):
-        >>>     # Represents a linear model of a simple pendulum:
-        ...     #  m * l**2 ω' + m * g * l * sin(θ) = 0
+        >>> def rhs(theta, omega, gravity, length, mass, omega_b, time):
+        ...     # Represents a linear model of a simple pendulum under
+        ...     # sinusoidal torquing.
+        ...     #  m * l**2 ω' + m * g * l * sin(θ) = sin(ω_b * t)
         ...     thetad = omega
-        ...     omegad = (-m*g*l*np.sin(theta)) / m / l**2
+        ...     omegad = (np.sin(omega_b * time) -
+        ...               m*g*l*np.sin(theta)) / m / l**2
         ...     return thetad, omegad  # in order of sys.states
         >>> sys.diff_eq_func = rhs
 
@@ -87,6 +95,7 @@ class MultiDoFNonLinearSystem(_System):
         msg = ('Your diff_eq_func does not return the correct number of '
                'state derivatives. Make sure the number and order of the '
                'states match the derivatives of the states you return.')
+
         res = self.diff_eq_func(*arg_vals)
         try:
             len(res)
@@ -99,36 +108,34 @@ class MultiDoFNonLinearSystem(_System):
     def _generate_array_rhs_eval_func(self):
 
         arg_names = getargspec(self.diff_eq_func).args
-        arg_vals = [self._get_par_vals(k) for k in arg_names]
 
         coord_names = list(self.coordinates.keys())
         speed_names = list(self.speeds.keys())
 
-        coord_idxs = [arg_names.index(n) for n in coord_names]
-        speed_idxs = [arg_names.index(n) for n in speed_names]
-
-        if 'time' in arg_names:
-            time_idx = arg_names.index('time')
-        else:
-            time_idx = None
+        n = len(speed_names)
 
         def eval_rhs(x, t):
             # x is either shape(1, 2n), shape(m, 2n), shape(1, 2n, 1) or shape(m, 2n, 1)
             # t is either float or shape(m,)
             # TODO : This could be slow for large # coords/speeds.
-            for i in range(len(coord_names)):
-                arg_vals[coord_idxs[i]] = x[:, i, ...]
-            for i in range(len(speed_names)):
-                arg_vals[speed_idxs[i]] = x[:, i + len(speed_names), ...]
-            if time_idx is not None:
-                if len(x.shape) == 3 and x.shape[-1] == 1:
-                    arg_vals[time_idx] = np.atleast_2d(t).T
-                else:
-                    arg_vals[time_idx] = np.asarray(t)
+            for i, cname in enumerate(coord_names):
+                self.coordinates[cname] = x[:, i, ...]
+            for i, sname in enumerate(speed_names):
+                self.speeds[sname] = x[:, i + n, ...]
+
+            if len(x.shape) == 3 and x.shape[-1] == 1:
+                self._time['t'] = np.atleast_2d(t).T
+            else:
+                self._time['t'] = np.asarray(t)
+
+            # this calculates the measurements
+            arg_vals = [self._get_par_vals(k) for k in arg_names]
+
             # TODO : Would be nice not to have to create this every eval.
             x_dot = np.zeros_like(x)
             for i, dot in enumerate(self.diff_eq_func(*arg_vals)):
                 x_dot[:, i, ...] = dot
+
             return x_dot
 
         return eval_rhs
@@ -141,6 +148,11 @@ class MultiDoFNonLinearSystem(_System):
         x0 = list(self.coordinates.values())
         v0 = list(self.speeds.values())
 
+        if len(x0) != len(v0):
+            msg = ('There is not and equal number of coordinates and speeds. '
+                   'Make sure you have added one speed for each coordinate.')
+            raise ValueError(msg)
+
         initial_conditions = np.hstack((x0, v0))
 
         method_name = '_integrate_with_{}'.format(integrator)
@@ -149,7 +161,12 @@ class MultiDoFNonLinearSystem(_System):
         # make sure rhs is up-to-date
         self._ode_eval_func = self._generate_array_rhs_eval_func()
 
-        return integrator_method(initial_conditions, times)
+        try:
+            traj = integrator_method(initial_conditions, times)
+        except:
+            raise
+        else:  # integration succeeds
+            return traj
 
     def _integrate_with_lsoda(self, initial_conditions, times):
         """This method should return the integration results in the form of
@@ -169,11 +186,6 @@ class MultiDoFNonLinearSystem(_System):
     def _integrate_with_rungakutta4(self, initial_conditions, times):
         """4th-order Runge-Kutta integration.
 
-        Parameters
-        ----------
-        initial_conditions : array_like, shape(n,)
-            Initial values of the state variables.
-        times : array, shape(m,)
             Array of time values at which to solve.
 
         Returns
@@ -215,15 +227,36 @@ class MultiDoFNonLinearSystem(_System):
         # m : num time samples
         # n : num coordinates/speeds
 
-        # rows correspond to time, columns to states (m x 2n x 1)
-        int_res = self._integrate_equations_of_motion(times,
-                                                      integrator=integrator)
+        # store values before integration
+        coords = self.coordinates.copy()
+        speeds = self.speeds.copy()
+        time = self._time['t']
 
-        assert int_res.shape == (len(times), len(self.states), 1)
+        try:
+            # rows correspond to time, columns to states (m x 2n x 1)
+            int_res = self._integrate_equations_of_motion(
+                times, integrator=integrator)
 
-        res = self._ode_eval_func(int_res, times)
+            if int_res.shape != (len(times), len(self.states), 1):
+                msg = ('Shape of trajectory from integration does not have '
+                       'the correct shape.')
+                raise ValueError(msg)
 
-        assert res.shape == (len(times), len(self.states), 1)
+            # calculate the accelerations
+            res = self._ode_eval_func(int_res, times)
+
+            if res.shape != (len(times), len(self.states), 1):
+                msg = ('Shape of derivatives does not have the correct shape.')
+                raise ValueError(msg)
+        except:
+            raise
+        finally:  # make sure to reset coords, speeds, time if anything fails
+            # reset to values before integration
+            for k, v in coords.items():
+                self.coordinates[k] = v
+            for k, v in speeds.items():
+                self.speeds[k] = v
+            self._time['t'] = time
 
         num_coords = len(self.coordinates)
         num_speeds = len(self.speeds)
