@@ -56,12 +56,9 @@ class _StatesDict(_collections.OrderedDict):
             raise ValueError(msg)
 
 
-class _CoordinatesDict(_collections.MutableMapping, dict):
+class _CoordinatesDict(_collections.OrderedDict):
     """A custom dictionary for storing coordinates and speeds."""
 
-    def __getitem__(self, key):
-        return dict.__getitem__(self, key)
-
     def __setitem__(self, key, value):
         # TODO : Shouldn't allow coordinates to be named with suffix _vel.
         if not key.isidentifier():
@@ -77,19 +74,7 @@ class _CoordinatesDict(_collections.MutableMapping, dict):
                    'Choose something different.')
             raise ValueError(msg.format(_FORBIDDEN_SUFFIXES))
         else:
-            dict.__setitem__(self, key, value)
-
-    def __delitem__(self, key):
-        dict.__delitem__(self, key)
-
-    def __iter__(self):
-        return dict.__iter__(self)
-
-    def __len__(self):
-        return dict.__len__(self)
-
-    def __contains__(self, x):
-        return dict.__contains__(self, x)
+            _collections.OrderedDict.__setitem__(self, key, value)
 
 
 class _SingleDoFCoordinatesDict(_CoordinatesDict):
@@ -149,6 +134,8 @@ class _MeasurementsDict(_collections.MutableMapping, dict):
         return func(*args)
 
     def __getitem__(self, key):
+        # TODO : This call to _check_for_duplicate_keys() is a big CPU time
+        # hog.
         self._check_for_duplicate_keys()
         return self._compute_value(key)
 
@@ -392,6 +379,7 @@ class System(object):
     def _get_par_vals(self, par_name):
         """Returns the value of any variable stored in the parameters,
         coordinates, or measurements dictionaries."""
+        # TODO : This get for duplicates is slow and called way too often.
         self._measurements._check_for_duplicate_keys()
         # TODO : This is duplicate of similar code in
         # _MeasurementsDict._compute_value(). Shouldn't have this redundancy.
@@ -463,7 +451,7 @@ class System(object):
 
     def _state_traj_to_dataframe(self, times, pos, vel, acc):
 
-        # pads with singleton dimension
+        # pads with singleton dimension if pos, vel, acc are 1D
         pos = np.atleast_2d(pos)
         vel = np.atleast_2d(vel)
         acc = np.atleast_2d(acc)
@@ -701,6 +689,8 @@ class System(object):
                     args.append(pop_list.pop(0))
             self.config_plot_update_func(*args)
 
+        # TODO : Run this with the initial conditions so that errors will
+        # propogate before the animation is run.
         # NOTE : This is useful to uncomment in debugging because the errors
         # push to the top if in the FuncAnimation.
         #gen_frame((1.0, self.result.iloc[0]), list(objs_to_modify))
